-------------------------------------------------------------------------
-- (C) COPYRIGHT International Business Machines Corp. 2000-2003
-- All Rights Reserved
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
-- dsyctldb.ddl; DB2 Everyplace Mobile Devices Control Database SQL.
-------------------------------------------------------------------------

--* Create database DSYCTLDB
create database DSYCTLDB;

--* Connect to database DSYCTLDB
connect to DSYCTLDB;

--* Update database DSYCTLDB performance parameters
update db config for dsyctldb using applheapsz 1024;
update db config for dsyctldb using logfilsiz 1024;
update db config for dsyctldb using logprimary 5;
update db config for dsyctldb using logsecond 5;

--* Create schema DSY (if does not exist)
create schema DSY;

--* Create table DSY.GROUPS (if does not exist)
create table
  DSY.GROUPS(
    ID char(13) for bit data primary key not null,
    NAME varchar(18) unique not null,
    DESCRIPTION varchar(128),
    ENABLED char(1) not null with default 'N',
    LDAP_NUMERICUNIQUEID BIGINT,
    LDAP_SECURITYNAME varchar(254),
    constraint CK_ENABLED check(ENABLED in('N','Y'))
  );
alter table DSY.GROUPS add column LDAP_NUMERICUNIQUEID BIGINT;
alter table DSY.GROUPS add column LDAP_SECURITYNAME varchar(254);

--* Create table DSY.CLIENTS (if does not exist)
create table
  DSY.CLIENTS(
    GROUP_ID char(13) for bit data,
    ID char(13) for bit data primary key not null,
    NAME varchar(254) unique not null,
    DESCRIPTION varchar(128),
    PASSWORD varchar(254),
    NEXTNONCE varchar(16),
    LDAP_NUMERICUNIQUEID BIGINT,
    LDAP_SECURITYNAME varchar(254),
    DB2E_USERID varchar(18),
    DB2E_PASSWORD varchar(18),
    constraint FK_GROUP_ID foreign key(GROUP_ID) references DSY.GROUPS(ID) on delete set null
  );
alter table DSY.CLIENTS add column NEXTNONCE varchar(16);
alter table DSY.CLIENTS add column LDAP_NUMERICUNIQUEID BIGINT;
alter table DSY.CLIENTS add column LDAP_SECURITYNAME varchar(254);
alter table DSY.CLIENTS add column DB2E_USERID varchar(18);
alter table DSY.CLIENTS add column DB2E_PASSWORD varchar(18);
alter table DSY.CLIENTS alter column NAME set data type varchar(254);
alter table DSY.CLIENTS alter column PASSWORD set data type varchar(254);

--* Create table DSY.DEVICES (if does not exist)
create table
  DSY.DEVICES(
    CLIENT_ID char(13) for bit data unique not null,
    ID varchar(56) not null,
    TYPE varchar(4) not null,
    ENABLED char(1) not null with default 'N',
    DIRTY char(1) not null with default 'Y',
    VERSION varchar(18),
    TIMESTAMP timestamp with default current timestamp,
    constraint FK_CLIENT_ID foreign key(CLIENT_ID) references DSY.CLIENTS(ID) on delete cascade,
    constraint UN_IDTYPE unique(ID,TYPE),
    constraint CK_ENABLED check(ENABLED in('N','Y')),
    constraint CK_DIRTY check(DIRTY in('N','Y'))
  );
alter table DSY.DEVICES add column VERSION varchar(18);
alter table DSY.DEVICES alter column ID set data type varchar(56);
alter table DSY.DEVICES add column TIMESTAMP timestamp with default current timestamp;

--* Create table DSY.DEVICECONFIG (if does not exist)
create table
  DSY.DEVICECONFIG(
    ID char(13) for bit data primary key not null,
    DEVICE_ID varchar(56) not null,
    DEVICE_TYPE varchar(4) not null,
    STATE char(1) not null with default '-',
    OBJECT_ID char(13) for bit data not null,
    OBJECT_NAME varchar(257) not null,
    OBJECT_TYPE varchar(1) not null,
    PARENT_OBJECT_ID char(13) for bit data,
    PARENT_OBJECT_NAME varchar(257),
    PARENT_OBJECT_TYPE varchar(1),
		 INSERT_TIMESTAMP timestamp with default current timestamp,
    constraint UN_DIDTYPE_OBJ unique(DEVICE_ID,DEVICE_TYPE,STATE,OBJECT_ID),
    constraint FK_DEVICE_ID_TYPE foreign key(DEVICE_ID,DEVICE_TYPE) references DSY.DEVICES(ID,TYPE) on delete cascade,
    constraint CK_STATE check(STATE in('A','U','D','-')),
    constraint CK_OBJTYPE check(OBJECT_TYPE in('G','U','A','S','T','C','I')),
    constraint CK_POBJTYPE check(PARENT_OBJECT_TYPE in('G','U','A','S','T','C','I'))
  );
alter table DSY.DEVICECONFIG alter column DEVICE_ID set data type varchar(56);
alter table DSY.DEVICECONFIG add column INSERT_TIMESTAMP timestamp with default current timestamp;

--* Create table DSY.DEVICECONFIGHISTORY (if does not exist)
create table
  DSY.DEVICECONFIGHISTORY(
    TIMESTAMP timestamp not null with default current timestamp,
    ID char(13) for bit data primary key not null,
    DEVICE_ID varchar(56) not null,
    DEVICE_TYPE varchar(4) not null,
    STATE char(1) not null with default '-',
    OBJECT_ID char(13) for bit data not null,
    OBJECT_NAME varchar(257) not null,
    OBJECT_TYPE varchar(1) not null,
    PARENT_OBJECT_ID char(13) for bit data,
    PARENT_OBJECT_NAME varchar(257),
    PARENT_OBJECT_TYPE varchar(1),
		 ORIG_INSERT_TIMESTAMP timestamp with default current timestamp
  );
alter table DSY.DEVICECONFIGHISTORY add column ORIG_INSERT_TIMESTAMP timestamp with default current timestamp;
create trigger DSY.DEVICECONFIGTRG after delete on DSY.DEVICECONFIG referencing old as OLD for each row mode db2sql begin atomic insert into DSY.DEVICECONFIGHISTORY values(current timestamp, OLD.ID, OLD.DEVICE_ID, OLD.DEVICE_TYPE, OLD.STATE, OLD.OBJECT_ID, OLD.OBJECT_NAME, OLD.OBJECT_TYPE, OLD.PARENT_OBJECT_ID, OLD.PARENT_OBJECT_NAME, OLD.PARENT_OBJECT_TYPE, OLD.INSERT_TIMESTAMP); end;

--* Create table DSY.APPLICATIONS (if does not exist)
create table
  DSY.APPLICATIONS(
    ID char(13) for bit data primary key not null,
    NAME varchar(18) unique not null,
    DESCRIPTION varchar(128),
    SIGNATURE varchar(18)
  );

--* Create table DSY.GROUP_APPLICATIONS (if does not exist)
-- no primary key as id's can exist many times
create table
  DSY.GROUP_APPLICATIONS(
    GROUP_ID char(13) for bit data not null,
    APPLICATION_ID  char(13) for bit data not null,
    ORDER integer with default 0,
    constraint UN_GROUP_APPL unique(GROUP_ID,APPLICATION_ID),
    constraint FK_GROUP_ID foreign key(GROUP_ID) references DSY.GROUPS(ID) on delete cascade,
    constraint FK_APPLICATION_ID foreign key(APPLICATION_ID) references DSY.APPLICATIONS(ID) on delete cascade
  );
alter table DSY.GROUP_APPLICATIONS add column ORDER integer with default 0;

--* Create table DSY.ADAPTERS (if does not exist)
create table
  DSY.ADAPTERS(
    ID char(13) for bit data primary key not null,
    NAME varchar(18) unique not null,
    DESCRIPTION varchar(128),
    FILE_ATTRIBUTES varchar(512),
    COMM_ATTRIBUTES varchar(512),
    SIGNATURE varchar(18),
    CLASSNAME varchar(512)
  );
alter table DSY.ADAPTERS add column CLASSNAME varchar(512);

--*   Insert default adapters (if do not exist)
insert into dsy.adapters(id, name, description, signature) values(x'00000000000000000000000000', 'DSYFILE',  'Default DB2 Everyplace File adapter',  'DSYFILE');
insert into dsy.adapters(id, name, description, signature) values(x'00000000000000000000000001', 'DSYDPROPR', 'Default DB2 Everyplace DataPropagator adapter', 'DSYDPROPR');
insert into dsy.adapters(id, name, description, signature) values(x'00000000000000000000000002', 'DSYJDBC', 'Default DB2 Everyplace JDBC adapter', 'DSYJDBC');
insert into dsy.adapters(id, name, description, file_attributes, comm_attributes, signature, classname) values(GENERATE_UNIQUE(), 'Agent Adapter', 'Default DB2 Everyplace Agent Adapter', 'libname=agent.dll', 'cmd=/servlet/com.ibm.mobileservices.adapter.agent;dest=127.0.0.1;8080;', 'DSYAGENT', 'AgentAdapter.class');
update dsy.adapters set name='DSYDPROPR' where name='DSYTABLE';
update dsy.adapters set signature='DSYDPROPR' where name='DSYDPROPR';
update dsy.adapters set description='Default DB2 Everyplace DataPropagator adapter' where name='DSYDPROPR';
delete from dsy.adapters where (signature='DSYPIM');

--* Create table DSY.SUBSCRIPTIONS (if does not exist)
create table
  DSY.SUBSCRIPTIONS(
    ID char(13) for bit data primary key not null,
    NAME varchar(18) unique not null,
    DESCRIPTION varchar(128),
    ADAPTER_ID char(13) for bit data,
    TIMESTAMP timestamp with default current timestamp,
    ENCRYPTION_LEVEL integer with default 0,
    BLOCK char(1) not null with default 'N',
    DEVICE_TYPES varchar(128),
    constraint CK_BLOCK check(BLOCK in('N','Y')),
    constraint FK_ADAPTER_ID foreign key(ADAPTER_ID) references DSY.ADAPTERS(ID) on delete set null
  );
alter table DSY.SUBSCRIPTIONS add column ENCRYPTION_LEVEL integer with default 0;
alter table DSY.SUBSCRIPTIONS add column BLOCK char(1) not null with default 'N';
alter table DSY.SUBSCRIPTIONS add column DEVICE_TYPES varchar(128);

--* Create table DSY.APPLICATION_SUBS (if does not exist)
create table
  DSY.APPLICATION_SUBS(
    APPLICATION_ID char(13) for bit data not null,
    SUBSCRIPTION_ID char(13) for bit data not null,
    ORDER integer with default 0,
    constraint UN_APPL_SUB unique(APPLICATION_ID,SUBSCRIPTION_ID),
    constraint FK_APPLICATION_ID foreign key(APPLICATION_ID) references DSY.APPLICATIONS(ID) on delete cascade,
    constraint FK_SUBSCRIPTION_ID foreign key(SUBSCRIPTION_ID) references DSY.SUBSCRIPTIONS(ID) on delete cascade
  );
alter table DSY.APPLICATION_SUBS add column ORDER integer with default 0;

--* Create table DSY.FILE_SUBSCRIPTIONS (if does not exist)
create table
  DSY.FILE_SUBSCRIPTIONS(
    SUBSCRIPTION_ID char(13) for bit data primary key not null,
    SOURCE varchar(255) not null,
    TARGET varchar(255),
    DIRTY char(1) not null with default 'Y',
    REFRESH_TIMESTAMP timestamp,
    constraint CK_DIRTY check(DIRTY in('N','Y')),
    constraint FK_SUBSCRIPTION_ID foreign key(SUBSCRIPTION_ID) references DSY.SUBSCRIPTIONS(ID) on delete cascade
  );

--* Create table DSY.REPL_MASTER (if does not exist)
create table
  DSY.REPL_MASTER(
    ID char(13) for bit data primary key not null,
    SYSTEM varchar(128) not null,
    INSTANCE varchar(8) not null,
    DATABASE varchar(18) not null,
    USERID varchar(128) not null,
    PASSWORD varchar(128) not null,
    DATABASEALIAS varchar(18),
    constraint UN_SID unique(SYSTEM,INSTANCE,DATABASE)
  );
alter table DSY.REPL_MASTER add column DATABASEALIAS varchar(18);

--* Create table DSY.REPL_MIRROR (if does not exist)
create table
  DSY.REPL_MIRROR(
    ID char(13) for bit data primary key not null,
    DATABASE varchar(18) not null,
    USERID varchar(128) not null,
    PASSWORD varchar(128) not null,
    SYNC_WINDOW integer with default 3600,
    SYNC_MAX_CLIENTS integer with default 10,
    constraint UN_DATABASE unique(DATABASE)
  );

--* Create table DSY.REPL_SUBSCRIPTIONS (if does not exist)
create table
  DSY.REPL_SUBSCRIPTIONS(
    SUBSCRIPTION_ID char(13) for bit data primary key not null,
    REPL_MASTER_ID char(13) for bit data not null,
    REPL_MIRROR_ID char(13) for bit data not null,
    TARGET_DATABASE varchar(18),
    APPLY_QUALIFIER char(18) not null,
    MIRROR_ONLY char(1) not null with default 'N',
    constraint FK_SUBSCRIPTION_ID foreign key(SUBSCRIPTION_ID) references DSY.SUBSCRIPTIONS(ID) on delete cascade,
    constraint FK_REPL_MSTR_ID foreign key(REPL_MASTER_ID) references DSY.REPL_MASTER(ID) on delete cascade,
    constraint FK_REPL_MIRR_ID foreign key(REPL_MIRROR_ID) references DSY.REPL_MIRROR(ID) on delete cascade,
    constraint CK_MIRRORONLY check(MIRROR_ONLY in('N','Y'))
  );

--* Create table DSY.REPL_SUB_TABLES (if does not exist)
create table
  DSY.REPL_SUB_TABLES(
    SUBSCRIPTION_ID char(13) for bit data not null,
    SOURCE_SCHEMA varchar(128) not null,
    SOURCE_TABLE varchar(128) not null,
    TARGET_SCHEMA varchar(128) not null,
    TARGET_TABLE varchar(128) not null,
    DELETEAUTH char(1) not null with default 'Y',
    INSERTAUTH char(1) not null with default 'Y',
    UPDATEAUTH char(1) not null with default 'Y',
    WHERE_CLAUSE varchar(1024),
    SELECTAUTH char(1) not null with default 'Y',
    ID char(13) for bit data,
    WHERE_CLAUSE2 varchar(1024),
    ORDER integer with default 0,
    ENCRYPT char(1) not null with default 'N',
		 TABLESPACE varchar(32),
    JOINFILTER integer with default 2,
    constraint FK_SUBSCRIPTION_ID foreign key(SUBSCRIPTION_ID) references DSY.SUBSCRIPTIONS(ID) on delete cascade,
    constraint UN_SUB_TBNAME unique(SUBSCRIPTION_ID,TARGET_SCHEMA,TARGET_TABLE),
    constraint CK_DELETEAUTH check(DELETEAUTH in('N','Y')),
    constraint CK_INSERTAUTH check(INSERTAUTH in('N','Y')),
    constraint CK_UPDATEAUTH check(UPDATEAUTH in('N','Y')),
    constraint CK_SELECTAUTH check(SELECTAUTH in('N','Y')),
    constraint CK_ENCRYPT check(ENCRYPT in('N','Y')),
    constraint CK_JOINFILTER check(JOINFILTER in(0, 1, 2))
  );
alter table DSY.REPL_SUB_TABLES add column SELECTAUTH char(1) not null with default 'Y' add constraint CK_SELECTAUTH check(SELECTAUTH in('N','Y'));
alter table DSY.REPL_SUB_TABLES add column ID char(13) for bit data;
update dsy.repl_sub_tables set id=generate_unique() where (id is null);
alter table DSY.REPL_SUB_TABLES add column WHERE_CLAUSE2 varchar(1024);
alter table DSY.REPL_SUB_TABLES add column ORDER integer with default 0;
alter table DSY.REPL_SUB_TABLES add column ENCRYPT char(1) not null with default 'N' add constraint CK_ENCRYPT check(ENCRYPT in('N','Y'));
alter table DSY.REPL_SUB_TABLES add column TABLESPACE varchar(32);
alter table DSY.REPL_SUB_TABLES add column JOINFILTER integer with default 2 add constraint CK_JOINFILTER check(JOINFILTER in(0, 1, 2));

--* Create table DSY.JDBC_MASTER (if does not exist)
create table
  DSY.JDBC_MASTER(
    ID char(13) for bit data primary key not null,
    DATABASE varchar(255) not null,
    DRIVER varchar(255) not null,
    USERID varchar(128) not null,
    PASSWORD varchar(128) not null,
    constraint UN_MASTER unique(DATABASE,DRIVER)
  );

--* Create table DSY.JDBC_MIRROR (if does not exist)
create table
  DSY.JDBC_MIRROR(
    ID char(13) for bit data primary key not null,
    DATABASE varchar(255) not null,
    DRIVER varchar(255) not null,
    USERID varchar(128) not null,
    PASSWORD varchar(128) not null,
    SYNC_WINDOW integer with default 3600,
    SYNC_MAX_CLIENTS integer with default 10,
    constraint UN_MIRROR unique(DATABASE,DRIVER)
  );

--* Create table DSY.JDBC_SUBSCRIPTIONS (if does not exist)
create table
  DSY.JDBC_SUBSCRIPTIONS(
    SUBSCRIPTION_ID char(13) for bit data primary key not null,
    JDBC_MASTER_ID char(13) for bit data not null,
    JDBC_MIRROR_ID char(13) for bit data,
    TARGET_DATABASE varchar(18),
    constraint FK_SUBSCRIPTION_ID foreign key(SUBSCRIPTION_ID) references DSY.SUBSCRIPTIONS(ID) on delete cascade,
    constraint FK_JDBC_MSTR_ID foreign key(JDBC_MASTER_ID) references DSY.JDBC_MASTER(ID) on delete cascade
  );
alter table DSY.JDBC_SUBSCRIPTIONS drop constraint FK_JDBC_MIRR_ID;

--* Create table DSY.RDB_SUB_COLUMNS (if does not exist)
create table
  DSY.RDB_SUB_COLUMNS(
    ID char(13) for bit data primary key not null,
    REPL_SUB_TABLE_ID char(13) for bit data not null,
    SOURCE varchar(512) not null,
    TARGET varchar(128) not null,
    SUBSCRIBE char(1) not null with default 'N',
    REPLICATE char(1) not null with default 'N',
    KEY char(1) not null with default 'N',
    CONSTRNT varchar(1024),
    DATAFILTER varchar(18),
--*    AUTOKEY char(1) not null with default 'N',
    constraint UN_RPLIDTRG unique(REPL_SUB_TABLE_ID,TARGET),
    constraint CK_SUBSCRIBE check(SUBSCRIBE in('N','Y')),
    constraint CK_REPLICATE check(REPLICATE in('N','Y')),
    constraint CK_KEY check(KEY in('N','Y'))
--*    constraint CK_AUTOKEY check(AUTOKEY in('N','Y'))
--* not unique/primary key constraint FK_REPL_SUB_ID foreign key(REPL_SUB_TABLE_ID) references DSY.REPL_SUB_TABLES(ID) on delete cascade
  );
--* alter table DSY.REPL_SUB_COLUMNS add column AUTOKEY char(1) not null with default 'N' add constraint CK_AUTOKEY check(AUTOKEY in('N','Y'));

drop table DSY.JDBC_SUB_COLUMNS;

--* Create table DSY.RDB_SUB_INDEXES (if does not exist)
create table
  DSY.RDB_SUB_INDEXES(
    REPL_SUB_TABLE_ID char(13) for bit data not null,
    ID char(13) for bit data primary key not null,
    NAME varchar(18) not null unique,
    COLNAMES varchar(1024) not null
  );

--* Create table DSY.CUSTOM_SUBSCRIPTIONS (if does not exist)
create table
  DSY.CUSTOM_SUBSCRIPTIONS(
    SUBSCRIPTION_ID char(13) for bit data primary key not null,
    OTHER clob(4096),
    constraint FK_SUBSCRIPTION_ID foreign key(SUBSCRIPTION_ID) references DSY.SUBSCRIPTIONS(ID) on delete cascade
  );

--* Create table DSY.GROUP_PROPERTIES (if does not exist)
create table
  DSY.GROUP_PROPERTIES(
    GROUP_ID char(13) for bit data not null,
    NAME varchar(18) not null,
    VALUE varchar(36),
    constraint UN_GROUP_NAME unique(GROUP_ID,NAME),
    constraint FK_GROUP_ID foreign key(GROUP_ID) references DSY.GROUPS(ID) on delete cascade
  );

--* Create table DSY.CLIENT_PROPERTIES (if does not exist)
create table
  DSY.CLIENT_PROPERTIES(
    CLIENT_ID char(13) for bit data not null,
    NAME varchar(18) not null,
    VALUE varchar(36),
    constraint UN_CLIENT_NAME unique(CLIENT_ID,NAME),
    constraint FK_CLIENT_ID foreign key(CLIENT_ID) references DSY.CLIENTS(ID) on delete cascade
  );

--* Create table DSY.LOG (if does not exist)
create table
  DSY.LOG(
    ID char(13) for bit data primary key not null,
    TIMESTAMP timestamp not null with default current timestamp,
    MSGNUM varchar(8) not null,
    MSGTEXT varchar(512) not null,
    USERNAME varchar(254),
    SUBSCRIPTIONNAME varchar(18),
    DATABASENAME varchar(255),
    HOSTPORT varchar(270)
  );
alter table DSY.LOG add column USERNAME varchar(254);
alter table DSY.LOG add column SUBSCRIPTIONNAME varchar(18);
alter table DSY.LOG add column DATABASENAME varchar(255);
alter table DSY.LOG add column HOSTPORT varchar(270);
alter table DSY.LOG alter column USERNAME set data type varchar(254);
alter table DSY.LOG alter column DATABASENAME set data type varchar(255);

--* Drop table DSY.TABLEMAPPER (if it exists)
drop table dsy.tablemapper;

--* Create table DSY.SERVERS (if does not exist)
create table
  DSY.SERVERS(
    NAME varchar(128) primary key not null,
    DESCRIPTION VARCHAR(128),
    HOST varchar(128) not null,
    PORT INTEGER not null with default 8080,
    SYNCENABLED CHAR(1) not null with default 'N',
    REPLENABLED CHAR(1) not null with default 'N',
    BLOCKED CHAR(1) not null with default 'N',
    STATUS INTEGER not null with default 0,
    constraint UN_HOSTPORT unique(HOST,PORT),
    constraint CK_SYNCENABLED check(SYNCENABLED in('N','Y')),
    constraint CK_REPLENABLED check(REPLENABLED in('N','Y')),
    constraint CK_BLOCKED check(BLOCKED in('N','Y'))
  );

--* Create table DSY.CL_IMPL_MAP (if does not exist)
create table
  DSY.CL_IMPL_MAP(
    OBJECT_ID char(13) for bit data not null,
    TYPE int not null with default 0,
    CLASSNAME varchar(255),
    ENABLED char(1) not null with default 'Y',
    constraint UN_IDTYPE unique(OBJECT_ID,TYPE),
    constraint CK_ENABLED check(ENABLED in('N','Y'))
  );

--* Create table DSY.PROPERTIES (if does not exist)
create table
  DSY.PROPERTIES(
    TYPE varchar(255) NOT NULL,
    NAME varchar(255) NOT NULL,
    VALUE varchar(1024),
    DESCRIPTION varchar(1024),
    constraint UN_TYPENAME unique(TYPE,NAME)
  );
--* Refer to dsyproperties.ddl for properties that use to be in this .ddl file

--*a mirror database should not have duplicate target_schema and target_table.
drop trigger DSY.UNQ_MIRR_TBLS;
create trigger DSY.UNQ_MIRR_TBLS 
no cascade before insert on DSY.REPL_SUB_TABLES  
referencing NEW as N 
for each row mode db2sql 
when(exists(select jm.id from
	dsy.repl_sub_tables t, 
	dsy.jdbc_subscriptions js, 
	dsy.jdbc_mirror jm 
	where 
	t.target_schema=N.target_schema and t.target_table=N.target_table and
	t.subscription_id=js.subscription_id and js.jdbc_mirror_id=jm.id
	INTERSECT
	select jm.id from
	dsy.jdbc_subscriptions js,
	dsy.jdbc_mirror jm
	where
	js.subscription_id=N.subscription_id and js.jdbc_mirror_id=jm.id
	) 
or 
exists(select rm.id from
	dsy.repl_sub_tables t, 
	dsy.repl_subscriptions rs, 
	dsy.repl_mirror rm 
	where 
	t.target_schema=N.target_schema and t.target_table=N.target_table and
	t.subscription_id=rs.subscription_id and rs.repl_mirror_id=rm.id 
	INTERSECT
	select rm.id from
	dsy.repl_subscriptions rs,
	dsy.repl_mirror rm
	where
	rs.subscription_id=N.subscription_id and rs.repl_mirror_id=rm.id
	)
)
BEGIN ATOMIC
     signal sqlstate '99007' ('Duplicate tables in mirror'); END;


--*enforce unique tables for jdbc_mirror;
drop trigger DSY.UNQ_INS_JDBC_TBLS; 
create trigger DSY.UNQ_INS_JDBC_TBLS 
no cascade before insert on DSY.JDBC_SUBSCRIPTIONS   
referencing NEW as N 
for each row mode db2sql 
when(exists(select t.target_schema, t.target_table, jm.id from
	dsy.repl_sub_tables t, 
	dsy.jdbc_subscriptions js, 
	dsy.jdbc_mirror jm 
	where 
	t.subscription_id=js.subscription_id and js.jdbc_mirror_id=jm.id
	INTERSECT
	select t.target_schema, t.target_table, jm.id from
	dsy.repl_sub_tables t, 
	dsy.jdbc_mirror jm
	where
	N.subscription_id=t.subscription_id and N.jdbc_mirror_id=jm.id
	) 
)
BEGIN ATOMIC
     signal sqlstate '99007' ('Duplicate tables in jdbc mirror'); END;


--*enforce unique tables for repl_mirror;
drop trigger DSY.UNQ_INS_REPL_TBLS; 
create trigger DSY.UNQ_INS_REPL_TBLS 
no cascade before insert on DSY.REPL_SUBSCRIPTIONS   
referencing NEW as N 
for each row mode db2sql 
when(exists(select t.target_schema, t.target_table, rm.id from
	dsy.repl_sub_tables t, 
	dsy.repl_subscriptions rs, 
	dsy.repl_mirror rm 
	where 
	t.subscription_id=rs.subscription_id and rs.repl_mirror_id=rm.id
	INTERSECT
	select t.target_schema, t.target_table, rm.id from
	dsy.repl_sub_tables t, 
	dsy.repl_mirror rm
	where
	N.subscription_id=t.subscription_id and N.repl_mirror_id=rm.id
	)
)
BEGIN ATOMIC
     signal sqlstate '99007' ('Duplicate tables in DPropR mirror'); END;


--*enforce unique tables for jdbc_mirror;
drop trigger DSY.UNQ_UPD_JDBC_TBLS; 
create trigger DSY.UNQ_UPD_JDBC_TBLS 
no cascade before update of JDBC_MIRROR_ID on DSY.JDBC_SUBSCRIPTIONS 
referencing NEW as N OLD as O 
for each row mode db2sql 
when((not O.jdbc_mirror_id=N.jdbc_mirror_id) and 
	exists(select t.target_schema, t.target_table, jm.id from
	dsy.repl_sub_tables t, 
	dsy.jdbc_subscriptions js, 
	dsy.jdbc_mirror jm 
	where 
	t.subscription_id=js.subscription_id and js.jdbc_mirror_id=jm.id
	INTERSECT
	select t.target_schema, t.target_table, jm.id from
	dsy.repl_sub_tables t, 
	dsy.jdbc_mirror jm
	where
	N.subscription_id=t.subscription_id and N.jdbc_mirror_id=jm.id
	) 
) 
BEGIN ATOMIC 
     signal sqlstate '99007' ('Duplicate tables in jdbc mirror'); END;


--*enforce unique tables for repl_mirror;
drop trigger DSY.UNQ_UPD_REPL_TBLS; 
create trigger DSY.UNQ_UPD_REPL_TBLS 
no cascade before update of REPL_MIRROR_ID on DSY.REPL_SUBSCRIPTIONS 
referencing NEW as N OLD as O 
for each row mode db2sql 
when((not O.repl_mirror_id=N.repl_mirror_id) and 
	exists(select t.target_schema, t.target_table, rm.id from
	dsy.repl_sub_tables t, 
	dsy.repl_subscriptions rs, 
	dsy.repl_mirror rm 
	where 
	t.subscription_id=rs.subscription_id and rs.repl_mirror_id=rm.id
	INTERSECT
	select t.target_schema, t.target_table, rm.id from
	dsy.repl_sub_tables t, 
	dsy.repl_mirror rm
	where
	N.subscription_id=t.subscription_id and N.repl_mirror_id=rm.id
	)
)
BEGIN ATOMIC
     signal sqlstate '99007' ('Duplicate tables in DPropR mirror'); END;

--*create trigger to reject inserts if there would be duplicate;
drop trigger DSY.UNQ_GRP_TBLS_1; 
CREATE TRIGGER DSY.UNQ_GRP_TBLS_1 
NO CASCADE BEFORE  INSERT  ON DSY.GROUP_APPLICATIONS  
REFERENCING  NEW AS N  
FOR EACH ROW  MODE DB2SQL 
when(exists(
(select t.target_schema, t.target_table
from 
dsy.repl_sub_tables t, 
dsy.application_subs sa, 
dsy.group_applications ga
where 
t.subscription_id = sa.subscription_id and 
sa.application_id = ga.application_id and ga.group_id=N.group_id) 
INTERSECT
(select t.target_schema, t.target_table
from 
dsy.repl_sub_tables t, 
dsy.application_subs sa
where 
t.subscription_id = sa.subscription_id and 
sa.application_id=N.application_id)
))
BEGIN ATOMIC
     signal sqlstate '99008' ('Duplicate target tables in group'); END;

--*  A group should not have duplicate target_schema and target_table.
--*  The first exists finds all the groups that contain the given table,
--*  then it intersects this result with the groups of the subscription that we
--*  are trying to insert this table into. If intersection is not empty set, then this a duplicate.;
drop trigger DSY.UNQ_GRP_TBLS_2;
create trigger DSY.UNQ_GRP_TBLS_2 
no cascade before insert on DSY.REPL_SUB_TABLES  
referencing NEW as N 
for each row mode db2sql 
when(exists(select ga.group_id, t.target_schema, t.target_table from
	dsy.repl_sub_tables t, 
	dsy.application_subs sa,
	dsy.group_applications ga  
	where 
	t.target_schema=N.target_schema and t.target_table=N.target_table and
	t.subscription_id=sa.subscription_id and ga.application_id=sa.application_id 
	INTERSECT 
	select ga.group_id, N.target_schema, N.target_table from
	dsy.application_subs sa,
	dsy.group_applications ga   
	where
	sa.subscription_id=N.subscription_id and ga.application_id=sa.application_id) 
)	
BEGIN ATOMIC
     signal sqlstate '99008' ('Duplicate target tables in group'); END;


--*  A group should not have duplicate target_schema and target_table.
--*  The first select finds all the groups that contain the given subscriptions tables,
--*  then it intersects this result with the groups of the subscription that we
--*  are trying to add into this subscription set. 
--*  If intersection is not empty set, then this a duplicate subscription in this group.;
drop trigger DSY.UNQ_GRP_TBLS_3;
create trigger DSY.UNQ_GRP_TBLS_3 
no cascade before insert on DSY.APPLICATION_SUBS  
referencing NEW as N 
for each row mode db2sql 
when(exists(select ga.group_id, t.target_schema, t.target_table from
	dsy.repl_sub_tables t, 
	dsy.application_subs sa,
	dsy.group_applications ga  
	where t.subscription_id=sa.subscription_id and ga.application_id=sa.application_id 
	INTERSECT 
	select ga.group_id, t.target_schema, t.target_table from
	dsy.group_applications ga,
	dsy.repl_sub_tables t    
	where
	t.subscription_id=N.subscription_id and ga.application_id=N.application_id) 
)	
BEGIN ATOMIC
     signal sqlstate '99008' ('Duplicate target tables in group'); END;



--* mlp#9136
delete from dsy.rdb_sub_columns where(repl_sub_table_id not in (select id from dsy.repl_sub_tables));
delete from dsy.rdb_sub_indexes where(repl_sub_table_id not in (select id from dsy.repl_sub_tables));

--* Commit changes
commit;

--* Disconnect from database DSYCTLDB
disconnect DSYCTLDB;

--* Finished
