#!/bin/sh
#######################################################################
# (C) COPYRIGHT International Business Machines Corp. 2000-2004
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# NAME:		 installFix.sh
#
# FUNCTION:
#		 install fixes to DB2 Everyplace		 
#   
#
########################################################################
#

showUsage () {

echo "Usage:"
echo "   installfix.sh [ -noRedeployEAR ] [ -useAdminServer ] username password DSYINSTDIR WAS_HOME FIX_DIRECTORY NODENAME SERVERNAME [ WASUSERNAME ] [ WASPASSWORD ]"
echo "\n"

echo    "-noRedeployEAR is only to be used if updating the ear is not desired."
echo    "-useAdminServer is valid for WAS 5 only. This option is required for WAS ND environments."
echo    "WASUSERNAME and WASPASSWORD is used for WAS 5 only, and they are required if security is enabled."


exit 1

}

redeployEAR () {
    if [ ! "$NOREDEPLOY" = "true" ]; then

	echo "redeployEAR..."
	# WAS 5:
	if [ -f "$WASHOME/bin/wsadmin.sh" ]; then
		$WASHOME/bin/wsadmin.sh $CONNTYPE -user "$WASUSERNAME" -password "$WASPASSWORD" -profile "$DSYINSTDIR/Server/installableApps/common/dsyDB2eEarEnv.jacl" -f "$DSYINSTDIR/Server/installableApps/common/dsyInstallDB2eEAR.jacl" "$NODENAME" "$SERVERNAME" "$DSYINSTDIR"
	fi

	# WAS 4
	if [ -f "$WASHOME/bin/XMLConfig.sh" ]; then
		"$WASHOME/bin/XMLConfig.sh" -adminNodeName $NODENAME -import "$DSYINSTDIR/Server/installableApps/common/dsy_was40_install_module.xml" -substitute "node=$NODENAME;DSYINSTDIR=$DSYINSTDIR;WAS_ROOT=$WASHOME"
	fi

    fi
}

backup () {
	echo "backing up old files... "
	mkdir -p "$FIXDIR/backup/Server/bin"
	mkdir -p "$FIXDIR/backup/Server/classes"
	mkdir -p "$FIXDIR/backup/Server/lib"
	mkdir -p "$FIXDIR/backup/Server/properties"
	mkdir -p "$FIXDIR/backup/Server/installableApps/common"
	mkdir -p "$FIXDIR/backup/Server/sample"
	mkdir -p "$FIXDIR/backup/WAS"
	cp -rf "$DSYINSTDIR/Server/bin" "$FIXDIR/backup/Server"
	cp -rf "$DSYINSTDIR/Server/classes" "$FIXDIR/backup/Server"
	cp -rf "$DSYINSTDIR/Server/lib" "$FIXDIR/backup/Server"
	cp -rf "$DSYINSTDIR/Server/properties" "$FIXDIR/backup/Server"
	cp -rf "$DSYINSTDIR/Server/installableApps/common" "$FIXDIR/backup/Server/installableApps"
	cp -rf "$DSYINSTDIR/Server/sample" "$FIXDIR/backup/Server"
	
}

applyFixes() {
	echo "applyFixes..."
	cp -rf "$FIXDIR/Server" "$DSYINSTDIR"
	cp -rf "$FIXDIR/WAS" "$DSYINSTDIR"

	if [ `uname` = "Linux" ]; then
		chown -R "$USERNAME" "$DSYINSTDIR"
		chgrp -R bin "$DSYINSTDIR/Server/bin"
	else
		chown -hR "$USERNAME" "$DSYINSTDIR"
		chgrp -hR bin "$DSYINSTDIR/Server/bin"
	fi

	chmod 700 "$DSYINSTDIR"/Server/bin/*.sh
	chmod 400 "$DSYINSTDIR"/Server/bin/*.ddl

	# get DSYPATH to update root installation:
	. "$DSYINSTDIR/Server/bin/dsyprofile.sh"
	cp -rf "$FIXDIR/Server" "$DSYPATH"
	cp -rf "$FIXDIR/WAS" "$DSYPATH"

}

install () {

	backup
	umask 022
	applyFixes
	redeployEAR

	su - $USERNAME -c "$DSYINSTDIR/Server/bin/dsyctldb.sh $USERNAME $PASSWORD"
	su - $USERNAME -c "$DSYINSTDIR/Server/bin/dsyjdbcmigration.sh"

}


##################
#Start of script
##################

if [ ! `whoami` = "root" ]; then
	echo "This command must be run as root".
	showUsage
fi


if [ "$1" = "-noRedeployEAR" ]; then		 
	NOREDEPLOY="true"
	shift 1
fi

if [ "$1" = "-useAdminServer" ]; then
	CONNTYPE=""
	shift 1
else
	CONNTYPE="-conntype NONE"
fi


USERNAME=$1
PASSWORD=$2
DSYINSTDIR=$3
WASHOME=$4
FIXDIR=$5
NODENAME=$6
SERVERNAME=$7

WASUSERNAME=$8
WASPASSWORD=$9


if [ ! -d "$DSYINSTDIR" ]; then
	echo "DB2 Everyplace Instance directory [$DSYINSTDIR] not found."
	showUsage
fi

if [ ! -d "$WASHOME" ]; then
	echo "WebSphere Application Server installation directory [$WASHOME] noot found."		 
	showUsage
fi

if [ ! -d "$FIXDIR" ]; then
	echo "The fix directory [$FIXDIR] does not exist."
	showUsage
fi

if [ ! -n "$USERNAME" ]; then
	echo "database instance username not specified."

	showUsage
fi

if [ ! -n "$PASSWORD" ]; then
	echo "database instance password not specified."
	showUsage
fi


echo "- Please ensure that both the DB2 Everyplace Sync Server and MDAC are stopped"
echo "  before continuing."
echo "Press Enter to continue..."
read AVAR

install
echo "end of $0"
